@@ -10,7 +10,7 @@ Note: Series are specified via frequency dict mapping column names to frequencie
 from .schema import (
     BaseModelConfig, DFMConfig, DDFMConfig,
     BaseResult, DFMResult, DDFMResult,
-    DFMFitParams, DDFMFitParams, FitParams,  # FitParams is backward compatibility alias
+    DFMStateSpaceParams, DDFMStateSpaceParams,
 )
 # DEFAULT_BLOCK_NAME is imported lazily where needed to avoid circular imports
 from .constants import (
@@ -46,8 +46,6 @@ from .types import (
     ForecastResult,
     CoefficientMatrix,
     CovarianceMatrix,
-    IRFArray,
-    CompanionMatrix,
     SeriesID,
     Frequency,
     DatasetName,
@@ -58,7 +56,6 @@ from .types import (
     ResultDict,
     ForecastDict,
     MetricsDict,
-    IRFResultDict,
     CheckpointDict,
     ConfigDict,
     Device,
@@ -107,9 +104,8 @@ __all__ = [
     # 'DEFAULT_BLOCK_NAME',  # Removed to avoid circular import - import directly from functional.dfm_block
     # Model-specific configs (from schema.py)
     'DFMConfig', 'DDFMConfig',
-    # Parameter overrides
-    'DFMFitParams', 'DDFMFitParams',
-    'FitParams',  # Backward compatibility alias
+    # State-space parameters
+    'DFMStateSpaceParams', 'DDFMStateSpaceParams',
     # Model-specific results (from results.py)
     'DFMResult', 'DDFMResult',
     # Utilities
@@ -139,8 +135,6 @@ __all__ = [
     'ForecastResult',
     'CoefficientMatrix',
     'CovarianceMatrix',
-    'IRFArray',
-    'CompanionMatrix',
     'SeriesID',
     'Frequency',
     'DatasetName',
@@ -151,7 +145,6 @@ __all__ = [
     'ResultDict',
     'ForecastDict',
     'MetricsDict',
-    'IRFResultDict',
     'CheckpointDict',
     'ConfigDict',
     'Device',
