@@ -2,6 +2,7 @@
 
 import pytest
 import numpy as np
+from dfm_python.config.constants import DEFAULT_DTYPE
 
 
 class TestBuilder:
@@ -14,7 +15,7 @@ class TestBuilder:
         
         N = 3
         m = 2
-        C = np.eye(N, m, dtype=np.float32)
+        C = np.eye(N, m, dtype=DEFAULT_DTYPE)
         H = build_observation_matrix(C, factor_order=1, N=N)
         # Should be [C, I] where I is N x N identity
         assert H.shape == (N, m + N)
